@@ -554,7 +554,11 @@ static void getPackageCounts(FFstrbuf* baseDir, FFPackagesResult* packageCounts,
     if (!(options->disabled & FF_PACKAGES_FLAG_PACMAN_BIT)) packageCounts->pacman += getNumElements(baseDir, "/var/lib/pacman/local", true);
     if (!(options->disabled & FF_PACKAGES_FLAG_LPKGBUILD_BIT)) packageCounts->lpkgbuild += getNumElements(baseDir, "/opt/Loc-OS-LPKG/lpkgbuild/remove", false);
     if (!(options->disabled & FF_PACKAGES_FLAG_PKGTOOL_BIT)) packageCounts->pkgtool += getNumElements(baseDir, "/var/log/packages", false);
-    if (!(options->disabled & FF_PACKAGES_FLAG_RPM_BIT)) packageCounts->rpm += getSQLite3Int(baseDir, "/var/lib/rpm/rpmdb.sqlite", "SELECT count(*) FROM Packages", "rpm");
+    if (!(options->disabled & FF_PACKAGES_FLAG_RPM_BIT))
+    {
+        // `Sigmd5` is the only table that doesn't contain the virtual `gpg-pubkey` package
+        packageCounts->rpm += getSQLite3Int(baseDir, "/var/lib/rpm/rpmdb.sqlite", "SELECT count(*) FROM Sigmd5", "rpm");
+    }
     if (!(options->disabled & FF_PACKAGES_FLAG_SNAP_BIT)) packageCounts->snap += getSnap(baseDir);
     if (!(options->disabled & FF_PACKAGES_FLAG_XBPS_BIT)) packageCounts->xbps += getXBPS(baseDir, "/var/db/xbps");
     if (!(options->disabled & FF_PACKAGES_FLAG_BREW_BIT))
@@ -642,33 +646,30 @@ void ffDetectPackagesImpl(FFPackagesResult* result, FFPackagesOptions* options)
     #endif
 
     ffStrbufSet(&baseDir, &instance.state.platform.homeDir);
+
     if (!(options->disabled & FF_PACKAGES_FLAG_NIX_BIT))
     {
-        // check if ~/.nix-profile exists
-        FF_STRBUF_AUTO_DESTROY profilePath = ffStrbufCreateCopy(&baseDir);
-        ffStrbufAppendS(&profilePath, ".nix-profile");
-        if (ffPathExists(profilePath.chars, FF_PATHTYPE_DIRECTORY))
-        {
-            result->nixUser += getNixPackages(&baseDir, ".nix-profile");
-        }
+        // Count packages from $HOME/.nix-profile
+        result->nixUser += getNixPackages(&baseDir, ".nix-profile");
 
-        // check if $XDG_STATE_HOME/nix/profile exists
-        FF_STRBUF_AUTO_DESTROY stateDir = ffStrbufCreate();
-        const char* stateHome = getenv("XDG_STATE_HOME");
-        if(ffStrSet(stateHome))
+        // Check in $XDG_STATE_HOME/nix/profile
+        FF_STRBUF_AUTO_DESTROY stateHome = ffStrbufCreate();
+        const char* stateHomeEnv = getenv("XDG_STATE_HOME");
+        if (ffStrSet(stateHomeEnv))
         {
-            ffStrbufSetS(&stateDir, stateHome);
-            ffStrbufEnsureEndsWithC(&stateDir, '/');
+            ffStrbufSetS(&stateHome, stateHomeEnv);
+            ffStrbufEnsureEndsWithC(&stateHome, '/');
         }
         else
         {
-            ffStrbufSet(&stateDir, &instance.state.platform.homeDir);
-            ffStrbufAppendS(&stateDir, ".local/state/");
+            ffStrbufSet(&stateHome, &instance.state.platform.homeDir);
+            ffStrbufAppendS(&stateHome, ".local/state/");
         }
+        result->nixUser += getNixPackages(&stateHome, "nix/profile");
 
-        ffStrbufSet(&profilePath, &stateDir);
-        ffStrbufAppendS(&profilePath, "nix/profile");
-        result->nixUser += getNixPackages(&stateDir, "nix/profile");
+        // Check in /etc/profiles/per-user/$USER
+        FF_STRBUF_AUTO_DESTROY userPkgsDir = ffStrbufCreateStatic("/etc/profiles/per-user/");
+        result->nixUser += getNixPackages(&userPkgsDir, instance.state.platform.userName.chars);
     }
 
     if (!(options->disabled & FF_PACKAGES_FLAG_GUIX_BIT))
