@@ -26,7 +26,7 @@ static void formatKey(const FFLocalIpOptions* options, FFLocalIpResult* ip, uint
         ffStrbufClear(key);
         FF_PARSE_FORMAT_STRING_CHECKED(key, &options->moduleArgs.key, ((FFformatarg[]) {
             FF_FORMAT_ARG(index, "index"),
-            FF_FORMAT_ARG(ip->name, "name"),
+            FF_FORMAT_ARG(ip->name, "ifname"),
             FF_FORMAT_ARG(ip->mac, "mac"),
             FF_FORMAT_ARG(options->moduleArgs.keyIcon, "icon"),
         }));
@@ -295,7 +295,7 @@ void ffParseLocalIpJsonObject(FFLocalIpOptions* options, yyjson_val* module)
     yyjson_obj_foreach(module, idx, max, key_, val)
     {
         const char* key = yyjson_get_str(key_);
-        if(ffStrEqualsIgnCase(key, "type"))
+        if(ffStrEqualsIgnCase(key, "type") || ffStrEqualsIgnCase(key, "condition"))
             continue;
 
         if (ffJsonConfigParseModuleArgs(key, val, &options->moduleArgs))
@@ -419,8 +419,8 @@ void ffGenerateLocalIpJsonConfig(FFLocalIpOptions* options, yyjson_mut_doc* doc,
 
     if (defaultOptions.showType != options->showType)
     {
-        if (options->showType & FF_LOCALIP_TYPE_IPV4_BIT)
-            yyjson_mut_obj_add_bool(doc, module, "showIpv4", true);
+        if (!(options->showType & FF_LOCALIP_TYPE_IPV4_BIT))
+            yyjson_mut_obj_add_bool(doc, module, "showIpv4", false);
 
         if (options->showType & FF_LOCALIP_TYPE_IPV6_BIT)
             yyjson_mut_obj_add_bool(doc, module, "showIpv6", true);
@@ -446,8 +446,8 @@ void ffGenerateLocalIpJsonConfig(FFLocalIpOptions* options, yyjson_mut_doc* doc,
         if (options->showType & FF_LOCALIP_TYPE_COMPACT_BIT)
             yyjson_mut_obj_add_bool(doc, module, "compact", true);
 
-        if (options->showType & FF_LOCALIP_TYPE_DEFAULT_ROUTE_ONLY_BIT)
-            yyjson_mut_obj_add_bool(doc, module, "defaultRouteOnly", true);
+        if (!(options->showType & FF_LOCALIP_TYPE_DEFAULT_ROUTE_ONLY_BIT))
+            yyjson_mut_obj_add_bool(doc, module, "defaultRouteOnly", false);
 
         if (options->showType & FF_LOCALIP_TYPE_ALL_IPS_BIT)
             yyjson_mut_obj_add_bool(doc, module, "showAllIps", true);
@@ -525,7 +525,7 @@ void ffInitLocalIpOptions(FFLocalIpOptions* options)
     ffOptionInitModuleArg(&options->moduleArgs, "󰩟");
 
     options->showType = FF_LOCALIP_TYPE_IPV4_BIT | FF_LOCALIP_TYPE_PREFIX_LEN_BIT
-        #if !__ANDROID__ /*Permission denied*/ && !__OpenBSD__ /*Report invalid argument for some reason*/ && !__DragonFly__ /*Doesn't work*/
+        #if !__ANDROID__ /*Permission denied*/
             | FF_LOCALIP_TYPE_DEFAULT_ROUTE_ONLY_BIT
         #endif
     ;
