@@ -6,11 +6,9 @@
 #include "modules/memory/memory.h"
 #include "util/stringUtils.h"
 
-#define FF_MEMORY_NUM_FORMAT_ARGS 3
-
 void ffPrintMemory(FFMemoryOptions* options)
 {
-    FFMemoryResult storage;
+    FFMemoryResult storage = {};
     const char* error = ffDetectMemory(&storage);
 
     if(error)
@@ -29,6 +27,7 @@ void ffPrintMemory(FFMemoryOptions* options)
         ? 0
         : (double) storage.bytesUsed / (double) storage.bytesTotal * 100.0;
 
+    FFPercentageTypeFlags percentType = options->percent.type == 0 ? instance.config.display.percentType : options->percent.type;
     if(options->moduleArgs.outputFormat.length == 0)
     {
         ffPrintLogoAndKey(FF_MEMORY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT);
@@ -38,17 +37,17 @@ void ffPrintMemory(FFMemoryOptions* options)
         {
             FF_STRBUF_AUTO_DESTROY str = ffStrbufCreate();
 
-            if(instance.config.display.percentType & FF_PERCENTAGE_TYPE_BAR_BIT)
+            if(percentType & FF_PERCENTAGE_TYPE_BAR_BIT)
             {
-                ffPercentAppendBar(&str, percentage, options->percent);
+                ffPercentAppendBar(&str, percentage, options->percent, &options->moduleArgs);
                 ffStrbufAppendC(&str, ' ');
             }
 
-            if(!(instance.config.display.percentType & FF_PERCENTAGE_TYPE_HIDE_OTHERS_BIT))
+            if(!(percentType & FF_PERCENTAGE_TYPE_HIDE_OTHERS_BIT))
                 ffStrbufAppendF(&str, "%s / %s ", usedPretty.chars, totalPretty.chars);
 
-            if(instance.config.display.percentType & FF_PERCENTAGE_TYPE_NUM_BIT)
-                ffPercentAppendNum(&str, percentage, options->percent, str.length > 0);
+            if(percentType & FF_PERCENTAGE_TYPE_NUM_BIT)
+                ffPercentAppendNum(&str, percentage, options->percent, str.length > 0, &options->moduleArgs);
 
             ffStrbufTrimRight(&str, ' ');
             ffStrbufPutTo(&str, stdout);
@@ -56,12 +55,18 @@ void ffPrintMemory(FFMemoryOptions* options)
     }
     else
     {
-        FF_STRBUF_AUTO_DESTROY percentageStr = ffStrbufCreate();
-        ffPercentAppendNum(&percentageStr, percentage, options->percent, false);
-        FF_PRINT_FORMAT_CHECKED(FF_MEMORY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, FF_MEMORY_NUM_FORMAT_ARGS, ((FFformatarg[]){
-            {FF_FORMAT_ARG_TYPE_STRBUF, &usedPretty},
-            {FF_FORMAT_ARG_TYPE_STRBUF, &totalPretty},
-            {FF_FORMAT_ARG_TYPE_STRBUF, &percentageStr},
+        FF_STRBUF_AUTO_DESTROY percentageNum = ffStrbufCreate();
+        if (percentType & FF_PERCENTAGE_TYPE_NUM_BIT)
+            ffPercentAppendNum(&percentageNum, percentage, options->percent, false, &options->moduleArgs);
+        FF_STRBUF_AUTO_DESTROY percentageBar = ffStrbufCreate();
+        if (percentType & FF_PERCENTAGE_TYPE_BAR_BIT)
+            ffPercentAppendBar(&percentageBar, percentage, options->percent, &options->moduleArgs);
+
+        FF_PRINT_FORMAT_CHECKED(FF_MEMORY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, ((FFformatarg[]){
+            FF_FORMAT_ARG(usedPretty, "used"),
+            FF_FORMAT_ARG(totalPretty, "total"),
+            FF_FORMAT_ARG(percentageNum, "percentage"),
+            FF_FORMAT_ARG(percentageBar, "percentage-bar"),
         }));
     }
 }
@@ -86,7 +91,7 @@ void ffParseMemoryJsonObject(FFMemoryOptions* options, yyjson_val* module)
     yyjson_obj_foreach(module, idx, max, key_, val)
     {
         const char* key = yyjson_get_str(key_);
-        if(ffStrEqualsIgnCase(key, "type"))
+        if(ffStrEqualsIgnCase(key, "type") || ffStrEqualsIgnCase(key, "condition"))
             continue;
 
         if (ffJsonConfigParseModuleArgs(key, val, &options->moduleArgs))
@@ -111,7 +116,7 @@ void ffGenerateMemoryJsonConfig(FFMemoryOptions* options, yyjson_mut_doc* doc, y
 
 void ffGenerateMemoryJsonResult(FF_MAYBE_UNUSED FFMemoryOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
 {
-    FFMemoryResult storage;
+    FFMemoryResult storage = {};
     const char* error = ffDetectMemory(&storage);
 
     if(error)
@@ -125,30 +130,27 @@ void ffGenerateMemoryJsonResult(FF_MAYBE_UNUSED FFMemoryOptions* options, yyjson
     yyjson_mut_obj_add_uint(doc, obj, "used", storage.bytesUsed);
 }
 
-void ffPrintMemoryHelpFormat(void)
-{
-    FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_MEMORY_MODULE_NAME, "{1} / {2} ({3})", FF_MEMORY_NUM_FORMAT_ARGS, ((const char* []) {
-        "Used size",
-        "Total size",
-        "Percentage used",
-    }));
-}
+static FFModuleBaseInfo ffModuleInfo = {
+    .name = FF_MEMORY_MODULE_NAME,
+    .description = "Print system memory usage info",
+    .parseCommandOptions = (void*) ffParseMemoryCommandOptions,
+    .parseJsonObject = (void*) ffParseMemoryJsonObject,
+    .printModule = (void*) ffPrintMemory,
+    .generateJsonResult = (void*) ffGenerateMemoryJsonResult,
+    .generateJsonConfig = (void*) ffGenerateMemoryJsonConfig,
+    .formatArgs = FF_FORMAT_ARG_LIST(((FFModuleFormatArg[]) {
+        {"Used size", "used"},
+        {"Total size", "total"},
+        {"Percentage used (num)", "percentage"},
+        {"Percentage used (bar)", "percentage-bar"},
+    }))
+};
 
 void ffInitMemoryOptions(FFMemoryOptions* options)
 {
-    ffOptionInitModuleBaseInfo(
-        &options->moduleInfo,
-        FF_MEMORY_MODULE_NAME,
-        "Print system memory usage info",
-        ffParseMemoryCommandOptions,
-        ffParseMemoryJsonObject,
-        ffPrintMemory,
-        ffGenerateMemoryJsonResult,
-        ffPrintMemoryHelpFormat,
-        ffGenerateMemoryJsonConfig
-    );
-    ffOptionInitModuleArg(&options->moduleArgs);
-    options->percent = (FFColorRangeConfig) { 50, 80 };
+    options->moduleInfo = ffModuleInfo;
+    ffOptionInitModuleArg(&options->moduleArgs, "");
+    options->percent = (FFPercentageModuleConfig) { 50, 80, 0 };
 }
 
 void ffDestroyMemoryOptions(FFMemoryOptions* options)
