@@ -17,20 +17,37 @@ void ffPrintPackages(FFPackagesOptions* options)
         return;
     }
 
+    uint32_t nixAll = counts.nixDefault + counts.nixSystem + counts.nixUser;
+    uint32_t flatpakAll = counts.flatpakSystem + counts.flatpakUser;
+    uint32_t brewAll = counts.brew + counts.brewCask;
+    uint32_t guixAll = counts.guixSystem + counts.guixUser + counts.guixHome;
+    uint32_t hpkgAll = counts.hpkgSystem + counts.hpkgUser;
+    uint32_t amAll = counts.amSystem + counts.amUser;
+
     if(options->moduleArgs.outputFormat.length == 0)
     {
         ffPrintLogoAndKey(FF_PACKAGES_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT);
 
-        #define FF_PRINT_PACKAGE_NAME(var, name) \
+        #define FF_PRINT_PACKAGE_NAME(var, name) {\
             if(counts.var > 0) \
             { \
                 printf("%u (%s)", counts.var, (name)); \
                 if((all -= counts.var) > 0) \
                     fputs(", ", stdout); \
-            };
+            } \
+        }
 
         #define FF_PRINT_PACKAGE(name) FF_PRINT_PACKAGE_NAME(name, #name)
 
+        #define FF_PRINT_PACKAGE_ALL(name) {\
+            if(name ## All > 0) \
+            { \
+                printf("%u (%s)", name ## All, #name); \
+                if((all -= name ## All) > 0) \
+                    fputs(", ", stdout); \
+            } \
+        }
+
         uint32_t all = counts.all;
         if(counts.pacman > 0)
         {
@@ -45,19 +62,47 @@ void ffPrintPackages(FFPackagesOptions* options)
         FF_PRINT_PACKAGE(emerge)
         FF_PRINT_PACKAGE(eopkg)
         FF_PRINT_PACKAGE(xbps)
-        FF_PRINT_PACKAGE_NAME(nixSystem, "nix-system")
-        FF_PRINT_PACKAGE_NAME(nixUser, "nix-user")
-        FF_PRINT_PACKAGE_NAME(nixDefault, "nix-default")
+        if (options->combined)
+        {
+            FF_PRINT_PACKAGE_ALL(nix);
+        }
+        else
+        {
+            FF_PRINT_PACKAGE_NAME(nixSystem, "nix-system")
+            FF_PRINT_PACKAGE_NAME(nixUser, "nix-user")
+            FF_PRINT_PACKAGE_NAME(nixDefault, "nix-default")
+        }
         FF_PRINT_PACKAGE(apk)
         FF_PRINT_PACKAGE(pkg)
         FF_PRINT_PACKAGE(pkgsrc)
-        FF_PRINT_PACKAGE_NAME(hpkgSystem, counts.hpkgUser ? "hpkg-system" : "hpkg")
-        FF_PRINT_PACKAGE_NAME(hpkgUser, "hpkg-user")
-        FF_PRINT_PACKAGE_NAME(flatpakSystem, counts.flatpakUser ? "flatpak-system" : "flatpak")
-        FF_PRINT_PACKAGE_NAME(flatpakUser, "flatpak-user")
+        if (options->combined)
+        {
+            FF_PRINT_PACKAGE_ALL(hpkg)
+        }
+        else
+        {
+            FF_PRINT_PACKAGE_NAME(hpkgSystem, counts.hpkgUser ? "hpkg-system" : "hpkg")
+            FF_PRINT_PACKAGE_NAME(hpkgUser, "hpkg-user")
+        }
+        if (options->combined)
+        {
+            FF_PRINT_PACKAGE_ALL(flatpak);
+        }
+        else
+        {
+            FF_PRINT_PACKAGE_NAME(flatpakSystem, counts.flatpakUser ? "flatpak-system" : "flatpak")
+            FF_PRINT_PACKAGE_NAME(flatpakUser, "flatpak-user")
+        }
         FF_PRINT_PACKAGE(snap)
-        FF_PRINT_PACKAGE(brew)
-        FF_PRINT_PACKAGE_NAME(brewCask, "brew-cask")
+        if (options->combined)
+        {
+            FF_PRINT_PACKAGE_ALL(brew);
+        }
+        else
+        {
+            FF_PRINT_PACKAGE_NAME(brew, "brew")
+            FF_PRINT_PACKAGE_NAME(brewCask, "brew-cask")
+        }
         FF_PRINT_PACKAGE(macports)
         FF_PRINT_PACKAGE(scoop)
         FF_PRINT_PACKAGE(choco)
@@ -65,28 +110,39 @@ void ffPrintPackages(FFPackagesOptions* options)
         FF_PRINT_PACKAGE(paludis)
         FF_PRINT_PACKAGE(winget)
         FF_PRINT_PACKAGE(opkg)
-        FF_PRINT_PACKAGE_NAME(amSystem, "am")
-        FF_PRINT_PACKAGE_NAME(amUser, "appman")
+        if (options->combined)
+        {
+            FF_PRINT_PACKAGE_ALL(am);
+        }
+        else
+        {
+            FF_PRINT_PACKAGE_NAME(amSystem, "am")
+            FF_PRINT_PACKAGE_NAME(amUser, "appman")
+        }
         FF_PRINT_PACKAGE(sorcery)
         FF_PRINT_PACKAGE(lpkg)
         FF_PRINT_PACKAGE(lpkgbuild)
-        FF_PRINT_PACKAGE_NAME(guixSystem, "guix-system")
-        FF_PRINT_PACKAGE_NAME(guixUser, "guix-user")
-        FF_PRINT_PACKAGE_NAME(guixHome, "guix-home")
+        if (options->combined)
+        {
+            FF_PRINT_PACKAGE_ALL(guix);
+        }
+        else
+        {
+            FF_PRINT_PACKAGE_NAME(guixSystem, "guix-system")
+            FF_PRINT_PACKAGE_NAME(guixUser, "guix-user")
+            FF_PRINT_PACKAGE_NAME(guixHome, "guix-home")
+        }
         FF_PRINT_PACKAGE(linglong)
         FF_PRINT_PACKAGE(pacstall)
         FF_PRINT_PACKAGE(mport)
         FF_PRINT_PACKAGE(qi)
+        FF_PRINT_PACKAGE(pisi)
+        FF_PRINT_PACKAGE(soar)
 
         putchar('\n');
     }
     else
     {
-        uint32_t nixAll = counts.nixDefault + counts.nixSystem + counts.nixUser;
-        uint32_t flatpakAll = counts.flatpakSystem + counts.flatpakUser;
-        uint32_t brewAll = counts.brew + counts.brewCask;
-        uint32_t guixAll = counts.guixSystem + counts.guixUser + counts.guixHome;
-        uint32_t hpkgAll = counts.hpkgSystem + counts.hpkgUser;
         FF_PRINT_FORMAT_CHECKED(FF_PACKAGES_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, ((FFformatarg[]){
             FF_FORMAT_ARG(counts.all, "all"),
             FF_FORMAT_ARG(counts.pacman, "pacman"),
@@ -128,6 +184,8 @@ void ffPrintPackages(FFPackagesOptions* options)
             FF_FORMAT_ARG(counts.pkgsrc, "pkgsrc"),
             FF_FORMAT_ARG(counts.hpkgSystem, "hpkg-system"),
             FF_FORMAT_ARG(counts.hpkgUser, "hpkg-user"),
+            FF_FORMAT_ARG(counts.pisi, "pisi"),
+            FF_FORMAT_ARG(counts.soar, "soar"),
             FF_FORMAT_ARG(nixAll, "nix-all"),
             FF_FORMAT_ARG(flatpakAll, "flatpak-all"),
             FF_FORMAT_ARG(brewAll, "brew-all"),
@@ -206,6 +264,7 @@ bool ffParsePackagesCommandOptions(FFPackagesOptions* options, const char* key,
                     FF_TEST_PACKAGE_NAME(PACMAN)
                     FF_TEST_PACKAGE_NAME(PACSTALL)
                     FF_TEST_PACKAGE_NAME(PALUDIS)
+                    FF_TEST_PACKAGE_NAME(PISI)
                     FF_TEST_PACKAGE_NAME(PKG)
                     FF_TEST_PACKAGE_NAME(PKGTOOL)
                     FF_TEST_PACKAGE_NAME(PKGSRC)
@@ -219,6 +278,7 @@ bool ffParsePackagesCommandOptions(FFPackagesOptions* options, const char* key,
                 case 'S': if (false);
                     FF_TEST_PACKAGE_NAME(SCOOP)
                     FF_TEST_PACKAGE_NAME(SNAP)
+                    FF_TEST_PACKAGE_NAME(SOAR)
                     FF_TEST_PACKAGE_NAME(SORCERY)
                     break;
                 case 'W': if (false);
@@ -242,6 +302,12 @@ bool ffParsePackagesCommandOptions(FFPackagesOptions* options, const char* key,
         return true;
     }
 
+    if (ffStrEqualsIgnCase(subKey, "combined"))
+    {
+        options->combined = ffOptionParseBoolean(value);
+        return true;
+    }
+
     return false;
 }
 
@@ -252,7 +318,7 @@ void ffParsePackagesJsonObject(FFPackagesOptions* options, yyjson_val* module)
     yyjson_obj_foreach(module, idx, max, key_, val)
     {
         const char* key = yyjson_get_str(key_);
-        if (ffStrEqualsIgnCase(key, "type"))
+        if (ffStrEqualsIgnCase(key, "type") || ffStrEqualsIgnCase(key, "condition"))
             continue;
 
         if (ffJsonConfigParseModuleArgs(key, val, &options->moduleArgs))
@@ -324,6 +390,7 @@ void ffParsePackagesJsonObject(FFPackagesOptions* options, yyjson_val* module)
                             FF_TEST_PACKAGE_NAME(PACMAN)
                             FF_TEST_PACKAGE_NAME(PACSTALL)
                             FF_TEST_PACKAGE_NAME(PALUDIS)
+                            FF_TEST_PACKAGE_NAME(PISI)
                             FF_TEST_PACKAGE_NAME(PKG)
                             FF_TEST_PACKAGE_NAME(PKGTOOL)
                             FF_TEST_PACKAGE_NAME(PKGSRC)
@@ -337,6 +404,7 @@ void ffParsePackagesJsonObject(FFPackagesOptions* options, yyjson_val* module)
                         case 'S': if (false);
                             FF_TEST_PACKAGE_NAME(SCOOP)
                             FF_TEST_PACKAGE_NAME(SNAP)
+                            FF_TEST_PACKAGE_NAME(SOAR)
                             FF_TEST_PACKAGE_NAME(SORCERY)
                             break;
                         case 'W': if (false);
@@ -352,6 +420,12 @@ void ffParsePackagesJsonObject(FFPackagesOptions* options, yyjson_val* module)
             }
         }
 
+        if (ffStrEqualsIgnCase(key, "combined"))
+        {
+            options->combined = yyjson_get_bool(val);
+            continue;
+        }
+
         ffPrintError(FF_PACKAGES_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", key);
     }
 }
@@ -388,6 +462,7 @@ void ffGeneratePackagesJsonConfig(FFPackagesOptions* options, yyjson_mut_doc* do
         FF_TEST_PACKAGE_NAME(PACMAN)
         FF_TEST_PACKAGE_NAME(PACSTALL)
         FF_TEST_PACKAGE_NAME(PALUDIS)
+        FF_TEST_PACKAGE_NAME(PISI)
         FF_TEST_PACKAGE_NAME(PKG)
         FF_TEST_PACKAGE_NAME(PKGTOOL)
         FF_TEST_PACKAGE_NAME(PKGSRC)
@@ -395,11 +470,15 @@ void ffGeneratePackagesJsonConfig(FFPackagesOptions* options, yyjson_mut_doc* do
         FF_TEST_PACKAGE_NAME(RPM)
         FF_TEST_PACKAGE_NAME(SCOOP)
         FF_TEST_PACKAGE_NAME(SNAP)
+        FF_TEST_PACKAGE_NAME(SOAR)
         FF_TEST_PACKAGE_NAME(SORCERY)
         FF_TEST_PACKAGE_NAME(WINGET)
         FF_TEST_PACKAGE_NAME(XBPS)
         #undef FF_TEST_PACKAGE_NAME
     }
+
+    if (options->combined != defaultOptions.combined)
+        yyjson_mut_obj_add_bool(doc, module, "combined", options->combined);
 }
 
 void ffGeneratePackagesJsonResult(FF_MAYBE_UNUSED FFPackagesOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
@@ -445,6 +524,7 @@ void ffGeneratePackagesJsonResult(FF_MAYBE_UNUSED FFPackagesOptions* options, yy
     FF_APPEND_PACKAGE_COUNT(pacman)
     FF_APPEND_PACKAGE_COUNT(pacstall)
     FF_APPEND_PACKAGE_COUNT(paludis)
+    FF_APPEND_PACKAGE_COUNT(pisi)
     FF_APPEND_PACKAGE_COUNT(pkg)
     FF_APPEND_PACKAGE_COUNT(pkgtool)
     FF_APPEND_PACKAGE_COUNT(pkgsrc)
@@ -453,6 +533,7 @@ void ffGeneratePackagesJsonResult(FF_MAYBE_UNUSED FFPackagesOptions* options, yy
     FF_APPEND_PACKAGE_COUNT(rpm)
     FF_APPEND_PACKAGE_COUNT(scoop)
     FF_APPEND_PACKAGE_COUNT(snap)
+    FF_APPEND_PACKAGE_COUNT(soar)
     FF_APPEND_PACKAGE_COUNT(sorcery)
     FF_APPEND_PACKAGE_COUNT(winget)
     FF_APPEND_PACKAGE_COUNT(xbps)
@@ -508,6 +589,8 @@ static FFModuleBaseInfo ffModuleInfo = {
         {"Number of pkgsrc packages", "pkgsrc"},
         {"Number of hpkg-system packages", "hpkg-system"},
         {"Number of hpkg-user packages", "hpkg-user"},
+        {"Number of pisi packages", "pisi"},
+        {"Number of soar packages", "soar"},
         {"Total number of all nix packages", "nix-all"},
         {"Total number of all flatpak app packages", "flatpak-all"},
         {"Total number of all brew packages", "brew-all"},
@@ -522,6 +605,7 @@ void ffInitPackagesOptions(FFPackagesOptions* options)
     ffOptionInitModuleArg(&options->moduleArgs, "󰏖");
 
     options->disabled = FF_PACKAGES_DISABLE_LIST;
+    options->combined = false;
 }
 
 void ffDestroyPackagesOptions(FFPackagesOptions* options)
